#!/usr/bin/env python
# coding: utf-8

from __future__ import print_function

from datetime import datetime, timedelta
from multiprocessing import Pool
import os
import sys
import netCDF4

import matplotlib
matplotlib.use('Agg')

import smartcarb as sc
import config

import create_2d_field_maps




def create_2d_fields(input_filename, constant_filename, output_filename, overwrite=True):
    """\
    Create input data for calculating satellite swaths.

    input_filename: COSMO output for a given hour.
    constant_filename: COSMO output for constant fields

    output_filename: output filename for 2d fields
    """
    if not os.path.exists(input_filename):
        return 42

    if not overwrite and os.path.exists(output_filename):
        return 42

    tracers = config.tracers.copy()

    with netCDF4.Dataset(constant_filename) as nc:
        h = nc.variables['HHL'][:]

    with netCDF4.Dataset(input_filename) as nc:
        lon = nc.variables['lon'][:]
        lat = nc.variables['lat'][:]
        rlon = nc.variables['rlon'][:]
        rlat = nc.variables['rlat'][:]

        print(output_filename)
        sc.io.init_netcdf(output_filename, rlon, rlat, lon, lat)

        T = nc.variables['T'][:]
        p = nc.variables['P'][:]
        ps = nc.variables['PS'][:]

        clct = nc.variables['CLCT'][:]

        # write to netCDF 
        sc.io.append_variable(output_filename, 'CLCT', clct, attrs=None)

        # mass of dry air (kg/m3)
        mair = sc.conv.calculate_mair(p, ps, h)

        for tracer in tracers:
            xm = nc.variables[tracer][:]

            if tracer.startswith('CO2_'):
                ground = sc.conv.xm_to_xv(xm[0,-1], 'CO2')
                column = sc.conv.calculate_xco2(xm, mair)

            elif tracer.startswith('NOX_'):
                ground = sc.conv.xm_to_cm(xm[0,-1], p[0,-1], T[0,-1])
                ground = sc.conv.nox_to_no2(ground)
                ground = sc.conv.cm_to_xm(ground, p[0,-1], T[0,-1])
                ground = sc.conv.xm_to_xv(ground, 'NO2')

                tracer = tracer.replace('NOX_', 'NO2_')

                column = sc.conv.calculate_vcd(xm, p, T, h, 'NO2')

            elif tracer.startswith('CO_'):
                ground = sc.conv.xm_to_xv(xm[0,-1], 'CO')
                column = sc.conv.calculate_vcd(xm, p, T, h, 'CO')
            else:
                raise ValueError('Tracers "%s" not handled.' % tracer)


            # write ground concentrations
            sc.io.append_variable(
                output_filename, '%s_ground' % tracer,
                ground, attrs=None
            )

            # write columns
            if tracer.startswith('CO2'):
                sc.io.append_variable(
                    output_filename,
                    'X%s' % tracer,
                    column, attrs=None
                )
            else:
                sc.io.append_variable(
                    output_filename,
                    '%s_column' % tracer,
                    column, attrs=None
                )



def create_multi_input(args):
    res = create_2d_fields(*args, overwrite=True)

    #if res == 42:
    #    return

    output_filename = args[-1]
    create_2d_field_maps.main(output_filename)


def create_args(starttime, hstart, hstop, output_path=config.path_2d_fields):
    """\
    startime (datetime)
    hstart (hour of simulation start)
    hstop (hour simulation stop)
    """
    args = []

    start = starttime + timedelta(hours=hstart)
    end = starttime + timedelta(hours=hstop)

    input_path = os.path.join(
        config.ROOT, 'mother',
        '%s_%d_%d' % (starttime.strftime('%Y%m%d%H'), hstart, hstop),
        'cosmo', 'output'
    )
    for time in sc.misc.iter_times(start, end, timedelta(hours=1)):
        args.append([
            os.path.join(input_path, time.strftime('lffd%Y%m%d%H.nc')),
            os.path.join(input_path, starttime.strftime('lffd%Y%m%d%Hc.nc')),
            os.path.join(output_path, time.strftime('cosmo_2d_%Y%m%d%H.nc'))
        ])

    return args


if __name__ == '__main__':
    starttimes = [datetime(2015,7,1)]#, datetime(2015,1,1)]
    hstarts = [144, 216]#, 288] # [0,   72, 144, 216]
    hstops =  [216, 288]#, 360] # [72, 144, 216, 288]

    os.makedirs(config.path_2d_fields, exist_ok=True)

    # create arguments
    args = []
    for starttime in starttimes:
        for hstart, hstop in zip(hstarts, hstops):
            args += create_args(starttime, hstart, hstop)

    # create pool for multiprocessing
    pool = Pool(processes=config.nproc)
    pool.map(create_multi_input, args)



